#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>

#include <rox/options.h>
#include <rox/choices.h>
#include <rox/rox_filer_action.h>
#include "menu_images.h"
#include "utils.h"

#include "Icons/group.xpm"
#include "Icons/null.xpm"
#include "Icons/file.xpm"

typedef struct {
	gchar *type;
	gchar *subtype;
	GdkPixbuf *image;
} MIMEImageEntry;

typedef struct {
	gchar *media;
	GdkPixbuf *image;
	GList *subtypes;
} MediaImageEntry;

typedef struct {
	gchar *subtype;
	GdkPixbuf *image;
} SubtypeImageEntry;

/*static GList *mime_image_list;*/

static gchar *get_mime_type_image_path(const char *media, const char* subtype);
static gchar *get_icon_path_by_file_path(const char *file);
#if 0
static gchar *get_media_type(const char *mimetype);
static gchar *get_subtype(const char *mimetype);
static MIMEImageEntry *mime_image_entry_new(const gchar *type);
static MIMEImageEntry *mime_image_entry_alloc(void);
static void mime_image_entry_destroy(MIMEImageEntry *ent);
static MediaImageEntry *media_entry_alloc(void);
static SubtypeImageEntry *subtype_entry_alloc(void);
static void media_entry_dealloc(MediaImageEntry *ent);
static void subtype_entry_dealloc(SubtypeImageEntry *ent);
static GtkWidget *get_mime_image_from_path(const char *path);
static GdkPixbuf *lookup_mime_image(const char *media, const char *subtype);
#endif
/*static void add_to_image_list(const char *media, const char *sybtype);*/

#if 0
static GdkPixbuf *lookup_mime_image(const char *media, const char *subtype) {
	GList *l = mime_image_list;
	MediaImageEntry *media_type;
	GdkPixbuf *ret = NULL;
	
	g_return_val_if_fail(media != NULL, NULL);
	while (l != NULL) {
		media_type = l->data;
		if (strcmp(media_type->media, media) == 0) break;
		else l = g_list_next(l);
	}
	return ret;
}

static gchar *get_subtype(const char *mimetype) {
	gchar *subtype, *ret;
	
	subtype = strstr(mimetype, G_DIR_SEPARATOR_S);
	
	if (subtype) subtype = subtype+1;
	else return NULL;
	
	ret = g_strdup(subtype);
	return ret;
}	

static gchar *get_media_type(const char *mimetype) {
	gchar *slash, *tmp, *ret;

	tmp = g_strdup(mimetype);
	slash = strstr(tmp, G_DIR_SEPARATOR_S);
	if (slash) *slash = 0;

	ret = g_strdup(tmp);
	g_free(tmp);
	return ret;
}

static MIMEImageEntry *mime_image_entry_new(const gchar *type) {
	char *media, *subtype, *fulltype;
	MIMEImageEntry *ret;
	
	ret = g_try_malloc(sizeof(MIMEImageEntry));
	g_return_val_if_fail(ret != NULL, NULL);
	
	fulltype = g_strdup(type);

	media = fulltype;
	subtype = strstr(fulltype, G_DIR_SEPARATOR_S);
	if (subtype) {
		*subtype = 0;
		subtype = subtype+1;
	}

	ret->type = g_strdup(media);
	if (subtype) ret->subtype = g_strdup(subtype);
	else ret->subtype = NULL;
	ret->image = NULL;
	g_free(fulltype);
	return ret;
}

static MIMEImageEntry *mime_image_entry_alloc(void) {
	MIMEImageEntry *ret;
	ret = g_try_malloc(sizeof(MIMEImageEntry));
	g_return_val_if_fail(ret != NULL, NULL);
	ret->type = NULL;
	ret->subtype = NULL;
	ret->image = NULL;
	return ret;
}

static MediaImageEntry *media_entry_alloc(void) {
	MediaImageEntry *ret;
	ret = g_try_malloc(sizeof(MediaImageEntry));
	g_return_val_if_fail(ret != NULL, NULL);
	ret->media = NULL;
	ret->image = NULL;
	ret->subtypes = NULL;
	return ret;
}

static void media_entry_dealloc(MediaImageEntry *ent) {
	GList *l, *rem;
	g_return_if_fail(ent != NULL);
	g_free(ent->media);
	g_object_unref(ent->image);
	l = ent->subtypes;
	while (l != NULL) {
		rem = l;
		l = g_list_remove_link(l, rem);
		subtype_entry_dealloc(rem->data);
		g_list_free_1(rem);
	}
	g_list_free(l);
}

static void subtype_entry_dealloc(SubtypeImageEntry *ent) {

}

static SubtypeImageEntry *subtype_entry_alloc(void) {
	SubtypeImageEntry *ret;
	ret = g_try_malloc(sizeof(SubtypeImageEntry));
	g_return_val_if_fail(ret != NULL, NULL);
	ret->subtype = NULL;
	ret->image = NULL;
	return ret;
}

static void mime_image_entry_destroy(MIMEImageEntry *ent) {
	g_return_if_fail(ent != NULL);
	g_free(ent->type);
	ent->type = NULL;
	if (ent->subtype) g_free(ent->subtype);
	ent->subtype = NULL;
	if (GDK_IS_PIXBUF(ent->image)) g_object_unref(ent->image);
	ent->image = NULL;
	g_free(ent);
}

/* Get the image for the file's MIME type from the list of images already loaded.  If 
   it is not found, add it to the list. */
static GtkWidget *get_mime_image_from_path(const char *path) {
	
	extern Option entry_width;
	extern Option entry_height;
	extern Option scale_up;
	
	gchar *mimetype, *media, *subtype, *image_path;
	GtkWidget *ret;
	GList *l = mime_image_list;
	int same_media=0, same_subtype=0;
	MIMEImageEntry *ent;

	g_return_val_if_fail(path != NULL, NULL);
	
	mimetype = rox_filer_file_type(path);
	media = get_media_type(mimetype);
	subtype = get_subtype(mimetype);

	while (l != NULL) {
		ent = l->data;
		if (l->data == NULL) {
			l = g_list_next(l);
			continue;
		}
		if (ent->type != NULL && media != NULL){
			same_media = strcmp(ent->type, media);}
		if (ent->subtype != NULL && subtype != NULL)
			same_subtype = strcmp(ent->subtype, subtype);
		if ( (same_media == 0 && same_subtype == 0) ) break;
		l = g_list_next(l);
		
	}

	if (l == NULL) {
		ent = mime_image_entry_alloc();
		ent->type = media;
		ent->subtype = subtype;
		image_path = get_mime_type_image_path(media, subtype);
		if (image_path) {
			ent->image = create_pixbuf(image_path, NULL, entry_width.int_value, 
			                           entry_height.int_value, scale_up.int_value);
		} else ent->image = NULL;
		mime_image_list = g_list_prepend(mime_image_list, ent);
		if (ent->image)
			ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(ent->image));
		else
			ret = NULL;
	} else {
		ent = l->data;
		if (ent->image) ret = gtk_image_new_from_pixbuf(ent->image);
		else ret = NULL;
		g_free(media);
		g_free(subtype);
	}
	return ret;
}
#endif

/* Gets the path for the image used for the specified MIME type.  Searches
   Choices/MIME-icons first and then the default ROX icon theme.  If the 
	particular MIME type is not found, defaults to the media type.  Currently
	does not honor ROX icon theme setting. */
static gchar *get_mime_type_image_path(const char *media, 
                                       const char* subtype) {
	gchar *ret = NULL;
	gchar *file_name, *file_path;

	if (subtype) file_name = g_strconcat(media, "_", subtype, ".png", NULL);
	else         file_name = g_strconcat(media, ".png", NULL);
	file_path = choices_find_path_load(file_name, "MIME-icons");

	if (!file_path) {
		g_free(file_name);
		if (subtype)
			file_name = g_strconcat("mime-", media, ":", subtype, ".png", NULL);
		else
			file_name = g_strconcat("mime-", media, ".png", NULL);
		file_path = g_build_path(G_DIR_SEPARATOR_S, g_getenv("HOME"), 
		                         ".icons", "ROX", "MIME", file_name, NULL);
		if (!g_file_test(file_path, G_FILE_TEST_EXISTS)) {
			g_free(file_path);
			ret = NULL;
		} else ret = file_path;
	} else ret = file_path;
	
	g_free(file_name);
	if (ret == NULL && subtype != NULL) ret = get_mime_type_image_path(media, NULL);
	
	return ret;
}

static gchar *get_icon_path_by_file_path(const char *file) {
	char *media, *subtype, *fulltype;
	gchar *ret;
	
	fulltype = rox_filer_file_type(file);

	media=fulltype;
	subtype = strstr(fulltype, G_DIR_SEPARATOR_S);
	if (subtype) {
		*subtype = 0;
		subtype = subtype+1;
	}

	ret = get_mime_type_image_path(media, subtype);
	g_free(fulltype);
	return ret;
}

/* This function adapted from the pixmaps.c file in ROX-Filer 2.0.1
   by Thomas Leonard. */
GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h, int uncond)
{
	int	w, h;
	w = gdk_pixbuf_get_width(src);
	h = gdk_pixbuf_get_height(src);

	if (w <= max_w && h <= max_h && !uncond)
	{
		gdk_pixbuf_ref(src);
		return src;
	}
	else
	{
		float scale_x = ((float) w) / max_w;
		float scale_y = ((float) h) / max_h;
		float scale = MAX(scale_x, scale_y);
		int dest_w = w / scale;
		int dest_h = h / scale;
		
		return gdk_pixbuf_scale_simple(src,
						MAX(dest_w, 1),
						MAX(dest_h, 1),
						GDK_INTERP_BILINEAR);
	}
}

GdkPixbuf *create_pixbuf(char *path, char ** default_icon, 
                         gint width, gint height, gint unconditional)
{
   GdkPixbuf *pixbuf, *pixbuf_scaled;

	g_return_val_if_fail(path != NULL, NULL);

   if (!access(path, R_OK)) {
		pixbuf = gdk_pixbuf_new_from_file(path, NULL);
   } else {
		pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) default_icon);
   }
	g_return_val_if_fail(pixbuf != NULL, NULL);
	pixbuf_scaled = scale_pixbuf(pixbuf, width, height, unconditional);
	
	/*if (GDK_IS_PIXBUF(pixbuf_scaled) && pixbuf != pixbuf_scaled) g_object_unref(pixbuf);*/
	g_object_unref(pixbuf);

	return pixbuf_scaled;
}

GtkWidget *create_pixmap(char *path, char ** default_icon, 
                         gint width, gint height, gint unconditional)
{
   GtkWidget *pixmap;
	GdkPixbuf *pixbuf;
	
	pixbuf = create_pixbuf(path, default_icon, width, height, unconditional);
	g_return_val_if_fail(pixbuf != NULL, NULL);
	pixmap = gtk_image_new_from_pixbuf(GDK_PIXBUF(pixbuf));
	
	/*if (GDK_IS_PIXBUF(pixbuf) ) g_object_unref(pixbuf);*/
	g_object_unref(pixbuf);

	return pixmap;
	
}

/* Takes a path and returns the GtkPixmap widget that will be used
   for the menu item. */
GtkWidget *get_menu_pixmap_path (gchar *path) {
	
	extern Option group_image;
	extern Option entry_image;
	extern Option file_image;
	extern Option entry_width;
	extern Option entry_height;
	extern Option scale_up;
	
	extern GdkPixbuf *default_group_pixmap;
	extern GdkPixbuf *default_entry_pixmap;
	extern GdkPixbuf *default_file_pixmap;
	
	gchar *icon_path;
	GtkWidget *ret;
	
	icon_path = g_build_path(G_DIR_SEPARATOR_S, path, ".DirIcon", NULL);
	
	if (is_appdir(path)) {
		
		if (!access((char*)icon_path, R_OK)) {
			ret = create_pixmap(icon_path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
		} else {
			/* No .DirIcon, check for AppIcon.xpm. */
			g_free(icon_path);
			icon_path = g_build_path(G_DIR_SEPARATOR_S, path, "AppIcon.xpm", NULL);
			if (!access((char*)icon_path, R_OK)) {
				ret = create_pixmap(icon_path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
			} else {
				/* No icon file.  Use default. */
				if (!default_entry_pixmap) {
					default_entry_pixmap = create_pixbuf(entry_image.value, 
					                    null_xpm, 
					                    entry_width.int_value, 
					                    entry_height.int_value, 
					                    scale_up.int_value);
				}
				ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(default_entry_pixmap));
			}
		}
		
	} else if (is_regdir(path)) {
		if (!access((char*)icon_path, R_OK)) {
			ret = create_pixmap(icon_path, NULL, entry_width.int_value, 
			                   entry_height.int_value, scale_up.int_value);
		} else {
			/* No .DirIcon, so clear icon_path. */
			if (!default_group_pixmap) {
					default_group_pixmap = create_pixbuf(group_image.value, 
					                    group_xpm, 
					                    entry_width.int_value, 
					                    entry_height.int_value, 
					                    scale_up.int_value);
			}
			ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(default_group_pixmap));
		}
		
	} else { /* Path is a regular file. */
		g_free(icon_path);
		icon_path = get_icon_path_by_file_path(path);
		/*ret = get_mime_image_from_path(path);*/
		/*if (ret == NULL) {*/
		if (icon_path != NULL) {
			ret = create_pixmap(icon_path, NULL, entry_width.int_value,
			                    entry_height.int_value, scale_up.int_value);
		} else {
			if (!default_file_pixmap) {
				default_file_pixmap = create_pixbuf(file_image.value,
				                          file_xpm,
				                          entry_width.int_value,
												  entry_height.int_value,
												  scale_up.int_value);
			}
			ret = gtk_image_new_from_pixbuf(GDK_PIXBUF(default_file_pixmap));
		}
	}
	g_free(icon_path);
	gtk_widget_show(ret);
	
	return ret;
}

void clear_image_list(void) {
#if 0
	GList *l = mime_image_list;
	GList *rem;
	while (l != NULL) {
		mime_image_entry_destroy(l->data);
		rem = l;
		l = g_list_remove_link(l, rem);
		g_list_free_1(rem);
	}
	if (mime_image_list) g_list_free(mime_image_list);
	if (mime_image_list!=NULL) printf("List not null\n");
	else printf("List IS null\n");
	mime_image_list = NULL;
#endif
}
